#   Copyright 2011 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

require 'logger'

class BaseHandler

  def initialize(logger=nil)
    logger(logger)
  end

  def handle(data)
    logger.debug "====== Type of event: #{data.event}"
    if data.event == 'STATUS'
      logger.debug "calling handle status in base handler"
      handle_status(data)
    elsif data.event == 'FAILURE'
      handle_failed(data)
    end
  end

  def handle_status(data)
    logger.debug "{data.event}, #{data.new_status}"
  end

  def handle_failed(data)
    logger.error "#{data.to_s}"
  end

  protected
  def logger(logger=nil)
    @logger ||= logger
    unless @logger
      @logger = Logger.new(STDOUT)
      @logger.level = Logger::ERROR
      @logger.datetime_format = "%Y-%m-%d %H:%M:%S"
    end
    return @logger
  end
end
