#   Copyright 2011 Red Hat, Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# TODO: figure out what I am doing wrong here that I need this line
$: << File.expand_path(File.join(File.dirname(__FILE__), "."))
require 'cqpid'
require 'qmf2'
require 'logger'
require 'base_handler'

class ImageFactoryConsole < Qmf2::ConsoleHandler

  attr_accessor :q, :handler

  def initialize(args={})
#    @retry_limit = args.include?(:retry_limit) ? args[:retry_limit] : 20
#    @delay = args.include?(:delay) ? args[:delay] : 15
    host = args.include?(:host) ? args[:host] : "localhost"
    port = args.include?(:port) ? args[:port] : 5672
#   url = "amqp://#{host}:#{port}" <- the amqp part here doesnt work yet
    url = "#{host}:#{port}"
    opts = {"reconnect"=>"true"}
    @connection = Cqpid::Connection.new(url, opts)
    @connection.open
    @session = Qmf2::ConsoleSession.new(@connection)
    @session.open
    @session.set_agent_filter("[and, [eq, _vendor, [quote, 'redhat.com']], [eq, _product, [quote, 'imagefactory']]]")

    if args.include?(:logger)
      @logger = args[:logger]
    else
      @logger = Logger.new(STDOUT)
      @logger.level = Logger::ERROR
      @logger.datetime_format = "%Y-%m-%d %H:%M:%S"
    end
    @handler = args.include?(:handler)? args[:handler]: BaseHandler.new(@logger)
    super(@session)
  end

  # Call this method to initiate a build, and get back an
  # array of BuildAdaptor objects.
  # * template => String
  # Description of what to build. Can be xml, uuid, or url
  # * targets => Array
  # Represents the names of the clouds to target (ec2, mock, rackspace, etc)
  # * image_id => String
  # The UUID of an image previously built
  # * build_id => String
  # The UUID of a previous build of the image
  # * Returns => an array of BuildAdaptor objects
  #
  def build(template, targets, image='', build='')
    targets = [targets] unless targets.instance_of?(Array)
    # TODO: return error if there is a problem calling this method or getting
    # a factory instance
    begin
      response = factory.build_image(image, build, template, targets)
      build_adaptors(response)
    rescue Exception => e
      @logger.debug "Encountered error in build_image: #{e}"
      return e
    end
  end

  # Call this method to push an image to a provider, and get back an
  # array of BuildAdaptor objects.
  # * providers => Array
  # Represents the target providers to build for (ec2-us-east, mock1, etc)
  # * credentials => String
  # XML block to be used for registration, upload, etc
  # * image_id => String
  # The UUID of an image previously built
  # * build_id => String
  # The UUID of a previous build of the image
  # * Returns => an array of BuildAdaptor objects
  #
  def push(providers, credentials, image_id, build_id='')
    providers = [providers] unless providers.instance_of?(Array)
    # TODO: return error if there is a problem calling this method or getting
    # a factory instance
    begin
      response = factory.push_image(image_id, build_id, providers, credentials)
      build_adaptors(response)
    rescue Exception => e
      @logger.debug "Encountered error in push_image: #{e}"
      return e
    end
  end

# Call this method to import an image
# array of BuildAdaptor objects.
# * image => string
# The uuid of an existing image
# * build => string
# the uuid of an existing build, or the empty string
# * target_identifier => string
# the target specific image ID
# * image_desc => string
# an xml string description of the image
# * target => string
# name of the cloud to target
# * provider => string
# the name of the cloud provider, often a region
# Returns => a map including UUIDs as strings for 'target_image', 'build', 'image', 'provider_image'

  def import_image(image, build, target_identifier, image_description, target, provider)
    # TODO: return error if there is a problem calling this method or getting
    # a factory instance
    begin
      factory.import_image(image, build, target_identifier, image_description, target, provider)
    rescue Exception => e
      @logger.debug "Encountered error in import_image: #{e}"
      return e
    end
  end

  # <b>DEPRECATED:</b> Please use <tt>build</tt> instead.
  # Call this method to initiate a build, and get back an
  # BuildAdaptor object.
  # * descriptor => String
  # This can be either xml or a url pointing to the xml
  # * target => String
  # Represents the target provider type to build for (ec2, mock, etc)
  # * Returns => a BuildAdaptor object
  #
  def build_image(descriptor, target)
    @logger.warn "[DEPRECATION] 'build_image' is deprecated.  Please use 'build' instead."
    # TODO: return error if there is a problem calling this method or getting
    # a factory instance
    begin
      response = factory.image(descriptor, target)
      build_adaptor(response)
    rescue Exception => e
      @logger.debug "Encountered error in build_image: #{e}"
      return e
    end
  end

  # <b>DEPRECATED:</b> Please use <tt>push</tt> instead.
  # Call this method to push an image to a provider, and get back an
  # BuildAdaptor object.
  # * image_id => String (uuid)
  # * provider => String
  # Represents the target provider to build for (ec2-us-east, mock, etc)
  # * credentials => String
  # XML block to be used for registration, upload, etc
  # * Returns => a BuildAdaptor object
  #
  def push_image(image_id, provider, credentials)
    @logger.warn "[DEPRECATION] 'push_image' is deprecated.  Please use 'push' instead."
    # TODO: return error if there is a problem calling this method or getting
    # a factory instance
    begin
      response = factory.provider_image(image_id, provider, credentials)
      build_adaptor(response)
    rescue Exception => e
      @logger.debug "Encountered error in push_image: #{e}"
      return e
    end
  end

  #TODO: enhance both of these methods to handle multiple agents
  def agent_added(agent)
    @logger.debug "GOT AN AGENT:  #{agent} at #{Time.now.utc}"
    @q = agent if agent.product == "imagefactory"
  end

  def agent_deleted(agent, reason)
    @logger.debug "AGENT GONE:  #{agent} at #{Time.now.utc}, because #{reason}"
    unless @q==nil
      @q = nil if (@q.product == agent.product && @q.name.eql?(agent.name))
    end
  end

  # TODO: handle agent restart events (this will be more useful when
  # restarted agent can recover an in-process build
  def agent_restarted(agent)
    @logger.debug "AGENT RESTARTED:  #{agent.product}"
  end

  # TODO: handle schema updates.  This will be more useful when/if
  # we make this a more generic console to talk to different agents.
  def agent_schema_updated(agent)
    @logger.debug "AGENT SCHEMA UPDATED:  #{agent.product}"
  end

  def event_raised(agent, data, timestamp, severity)
    @logger.debug "GOT AN EVENT:  #{agent}, #{data} at #{timestamp}"
    @handler.handle(data)
  end

  def shutdown
    @logger.debug "Closing connections.."
    if @session
      @session.close
    end
    @connection.close

    #TODO Replace this with some control flow to manage the cancelling of the console handler, atm
    #     this code swallows error message generated by closing connection, then cancelling console.
    begin
      self.cancel
    rescue QmfError
      # TODO We should be logging to a file rather than STDOUT, uncomment this code when logging is configured properly
      # Commented out to clean up user output
      #@logger.error "Issue Closing Console Handler"
    end
  end

  private

  def factory
    @factory ||= @q.query("{class:ImageFactory, package:'com.redhat.imagefactory'}").first
  end

  def build_adaptors(response)
    ret = []
    response['build_adaptors'].each do |imgfacaddr|
      query = Qmf2::Query.new(Qmf2::DataAddr.new(imgfacaddr))
      ret << @q.query(query,5).first
    end
    ret
  end

  def build_adaptor(response)
    @logger.warn "[DEPRECATION] 'build_adaptor' is deprecated.  Please use 'build_adaptors' instead."
    imgfacaddr = Qmf2::DataAddr.new(response['build_adaptor'])
    query = Qmf2::Query.new(imgfacaddr)
    @q.query(query,5).first
  end

  def make_array(a)
    return a ||= [a] unless a.instance_of?(Array)
  end
end
